from Acquisition import aq_inner

from collective.ogtags.browser.controlpanel import IOGTagsControlPanel
from collective.ogtags.interfaces import IOGTagsImageProvider
from plone.app.layout.viewlets import ViewletBase
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.utils import getSiteLogo
from Products.CMFPlone.utils import safe_unicode
from zope.component import ComponentLookupError
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter

try:
    from collective.behavior.seo.interfaces import ISEOFieldsMarker

    BEHAVIOR_SEO = True
except:
    BEHAVIOR_SEO = False


class OGTagsViewlet(ViewletBase):
    def meta_tags(self):
        try:
            self.settings = getUtility(IRegistry).forInterface(IOGTagsControlPanel)
        except (ComponentLookupError, KeyError):
            # Note: a KeyError happens when we have added a field to the
            # interface but haven't upgraded yet.
            return
        if not self.settings.enabled:
            return
        context = aq_inner(self.context)
        tags = {}

        # Basic properties
        title = context.title
        description = context.Description()
        url = context.absolute_url()

        # Allow overrides from collective.behavior.seo
        if BEHAVIOR_SEO and ISEOFieldsMarker.providedBy(context):
            if context.seo_title:
                title = safe_unicode(context.seo_title)
            if context.seo_description:
                description = safe_unicode(context.seo_description)

        # set title
        if title:
            tags['og:title'] = title
            tags['twitter:title'] = title

        # set description
        if description:
            tags['og:description'] = description
            tags['twitter:description'] = description

        # set url
        if url:
            tags['og:url'] = url

        # social media specific
        if self.settings.fb_id:
            tags['fb:app_id'] = self.settings.fb_id
        if self.settings.fb_username:
            tags['og:article:publisher'] = (
                'https://www.facebook.com/' + self.settings.fb_username
            )
        if self.settings.tw_id:
            tags['twitter:site'] = self.settings.tw_id
        tags['twitter:card'] = u'summary'

        # misc
        tags['og:type'] = u'website'
        if self.settings.og_site_name:
            tags['og:site_name'] = self.settings.og_site_name
        if self.settings.pinterest_id:
            tags['p:domain_verify'] = self.settings.pinterest_id

        return tags

    def image_tags(self):
        tags = []
        context = aq_inner(self.context)
        try:
            self.settings = getUtility(IRegistry).forInterface(IOGTagsControlPanel)
        except (ComponentLookupError, KeyError):
            return
        if not self.settings.enabled:
            return

        default_image = self.default_image(self.settings.default_img)
        try:
            image_provider = IOGTagsImageProvider(context)
        except TypeError:
            return default_image
        result = image_provider.getOGTagsImage()
        if result is None:
            return default_image
        image, fieldname = result
        try:
            scales = image.restrictedTraverse('@@images', None)
        except (AttributeError, KeyError):
            scales = None
        if not image or not fieldname or not scales:
            return default_image
        tag_scales = []
        for scale in ['og_fbl', 'og_fb', 'og_tw', 'og_ln']:
            try:
                image = scales.scale(fieldname, scale=scale)
                if not image:
                    continue
            except (AttributeError, TypeError) as e:
                continue
            tag = {}
            if scale == 'og_tw':
                tag['twitter:image'] = image.url
            else:
                if (image.width, image.height) not in tag_scales:
                    tag['og:image'] = image.url
                    tag['og:image:width'] = image.width
                    tag['og:image:height'] = image.height
                    tag_scales.append((image.width, image.height))
            tags.append(tag.copy())
        return tags or default_image

    def default_image(self, image):
        if image is not None:
            portal_state = getMultiAdapter(
                (self.context, self.request), name=u'plone_portal_state'
            )
            site_root_url = portal_state.navigation_root_url()
            image_url = '%s%s' % (site_root_url, image)
        else:
            image_url = getSiteLogo()

        twitter_tag = {}
        twitter_tag['twitter:image'] = image_url
        og_tag = {}
        og_tag['og:image'] = image_url
        return [twitter_tag, og_tag]



from zope.interface import Interface

class IOGTagsImageProvider(Interface):
    """Adapter to get the image that will be used by ogtags viewlet."""

    def getOGTagsImage(self):
        """ Get the context and Image fieldname that will be used by ogtags.
        to generate the scales.

        @return: a tuple in the form: (context, fieldname, ) or None.
        Context is most likely to be (like in the default adapters) the adapted
        context but custom implementations could return parent folders, the
        first Image portal_type contained by the current context or any other
        catalog search result that suits your needs.
        """



from collective.ogtags import MessageFactory as _
from plone.app.registry.browser import controlpanel
from zope import schema
from zope.interface import Interface


class IOGTagsControlPanel(Interface):
    enabled = schema.Bool(
        title=_(u'Enabled'),
        default=True)

    og_site_name = schema.TextLine(
        title=_(u'Site name'),
        required=False,
        default=u'')

    fb_username = schema.TextLine(
        title=_(u'Facebook username'),
        required=False,
        default=u'')

    fb_id = schema.TextLine(
        title=_(u'Facebook app id'),
        required=False,
        default=u'')

    tw_id = schema.TextLine(
        title=_(u'Twitter username'),
        description=_(u'example: @zestsoftware'),
        required=False,
        default=u'')

    default_img = schema.TextLine(
        title=_(u'Default image'),
        description=_(u'Path to default image'),
        required=False)

    pinterest_id = schema.TextLine(
        title=_(u'Pinterest id'),
        required=False,
        default=u'')


class OGTagsCPForm(controlpanel.RegistryEditForm):
    schema = IOGTagsControlPanel
    label = _(u'OGTags settings')
    description = _(u'Settings for collective.ogtags')


class OGTagsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = OGTagsCPForm



from zope.interface import implementer

from collective.ogtags.interfaces import IOGTagsImageProvider


@implementer(IOGTagsImageProvider)
class OGTagsImageDexterityProvider(object):

    def __init__(self, context):
        self.context = context

    def getOGTagsImage(self):
        """ Tries to return standard image fields named 'image' and 'lead image'
            Returns None in case of failure"""
        for fieldname in ["image", "leadImage"]:
            try:
                getattr(self.context, fieldname)
                return self.context, fieldname
            except AttributeError:
                pass
        return


@implementer(IOGTagsImageProvider)
class OGTagsImageArchetypeProvider(object):

    def __init__(self, context):
        self.context = context

    def getOGTagsImage(self):
        """ Tries to return standard image fields named 'image' and 'lead image'
            Returns None in case of failure"""
        for fieldname in ["image", "leadImage"]:
            image_field = self.context.getField(fieldname)
            if image_field:
                return self.context, fieldname
        return



# -*- extra stuff goes here -*-

from zope.i18nmessageid import MessageFactory

# your.app.package must match domain declaration in .po files
MessageFactory = MessageFactory('collective.ogtags')


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



from setuptools import setup, find_packages

version = '2.0.3'

setup(
    name='collective.ogtags',
    version=version,
    description='OpenGraph for plone5',
    long_description=(open('README.rst').read() + '\n' +
                      open('CONTRIBUTORS.rst').read() + '\n' +
                      open('CHANGES.rst').read()),
    # Get more strings from
    # https://pypi.python.org/pypi?:action=list_classifiers
    classifiers=[
        'Framework :: Plone',
        'Programming Language :: Python',
        'Framework :: Plone :: 5.1',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='OpenGraph tags facebook twitter linkedin',
    author='Diederik Veeze',
    author_email='d.veeze@zestsoftware.nl',
    url='https://zestsoftware.nl/',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['collective'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
    ],
    entry_points="""
    # -*- Entry points: -*-

    [z3c.autoinclude.plugin]
    target = plone
    """,
)
